
--
-- Name: master_tbl_eval_total_view; Type: VIEW; Schema: public; Owner: postgres
--

BEGIN;

CREATE OR REPLACE  VIEW master_tbl_eval_total_view AS
    SELECT m.* FROM master_tbl m JOIN ka_status_tbl s
        ON m.id = s.master_id
        WHERE
            s.status <> 3 AND
            (m.bearbeiter_id = (getuserid())::integer)
            OR pg_has_role('admin_ka','MEMBER');

GRANT SELECT ON master_tbl_eval_total_view TO :cm_ka_group;
GRANT SELECT ON master_tbl_eval_total_view TO :admin_ka_group;

--
-- Name: master_tbl_eval_incm_view; Type: VIEW; Schema: public; Owner: postgres
--

CREATE OR REPLACE  VIEW master_tbl_eval_incm_view AS
    SELECT * FROM master_tbl_eval_total_view
        WHERE phase IN (2,3,4,5);

GRANT SELECT ON public.master_tbl_eval_incm_view TO :cm_ka_group;
GRANT SELECT ON public.master_tbl_eval_incm_view TO :admin_ka_group;

CREATE OR REPLACE  VIEW master_tbl_incm_view AS
    SELECT * FROM master_tbl_view
        WHERE phase IN (2,3,4,5);

GRANT SELECT ON public.master_tbl_incm_view TO :cm_ka_group;
GRANT SELECT ON public.master_tbl_incm_view TO :admin_ka_group;

-- 
-- Sicht auf die Benutzerdatenbank
--

CREATE OR REPLACE VIEW ka_benutzer_tbl_view AS 
SELECT ka_benutzer_tbl.id, ka_benutzer_tbl.vorname, ka_benutzer_tbl.nachname,
ka_benutzer_tbl.telefon, ka_benutzer_tbl.aktiviert,
ka_benutzer_tbl.passwort_aendern, ka_benutzer_tbl.login,
ka_benutzer_tbl.raumnummer, ka_benutzer_tbl.filiale, p.usesysid,
ka_benutzer_tbl.vertreter, getrole(ka_benutzer_tbl.id) as rolle,
ka_benutzer_tbl.gid,
ka_benutzer_tbl.last_login
FROM ka_benutzer_tbl JOIN pg_user p
ON ka_benutzer_tbl.login = p.usename;

CREATE OR REPLACE RULE ka_benutzer_update_rule AS ON UPDATE TO ka_benutzer_tbl_view 
	DO INSTEAD  UPDATE ka_benutzer_tbl SET 
	vorname = new.vorname, 
	nachname = new.nachname, 
	telefon = new.telefon, 
	aktiviert = new.aktiviert, 
	raumnummer = new.raumnummer, 
	filiale = new.filiale, 
	passwort_aendern = new.passwort_aendern, 
	vertreter = new.vertreter,
	gid = new.gid,
	last_login = new.last_login
	WHERE ka_benutzer_tbl.id = new.id;

GRANT SELECT ON ka_benutzer_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_benutzer_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_benutzer_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE ka_benutzer_delete_rule AS ON DELETE TO ka_benutzer_tbl_view 
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_benutzer_insert_rule AS ON INSERT TO ka_benutzer_tbl_view
    DO INSTEAD NOTHING;

-- 
-- Sicht auf die Benutzer-Fallakte Transitionstabelle
--

CREATE OR REPLACE VIEW nm_benutzer_master_tbl_view AS 
SELECT nm_benutzer_master_tbl.id, nm_benutzer_master_tbl.benutzer_id,
nm_benutzer_master_tbl.master_id 
FROM nm_benutzer_master_tbl;

GRANT SELECT ON nm_benutzer_master_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON nm_benutzer_master_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE nm_benutzer_master_delete_rule AS ON DELETE TO 
nm_benutzer_master_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE nm_benutzer_master_insert_rule AS ON INSERT TO
nm_benutzer_master_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE nm_benutzer_master_update_rule AS ON UPDATE TO
nm_benutzer_master_tbl_view DO INSTEAD NOTHING;

-- Sichten auf die Benutzergruppe

CREATE OR REPLACE VIEW ka_benutzergruppe_tbl_view AS 
SELECT g.id, coalesce(b.nachname ||', '|| b.vorname, g.name) AS name FROM ka_benutzer_tbl b RIGHT JOIN ka_benutzergruppe_tbl g ON g.id = b.gid;

CREATE OR REPLACE RULE ka_benutzergruppe_delete_rule AS ON DELETE TO 
ka_benutzergruppe_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_benutzergruppe_insert_rule AS ON INSERT TO 
ka_benutzergruppe_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_benutzergruppe_update_rule AS ON UPDATE TO ka_benutzergruppe_tbl_view 
	DO INSTEAD  UPDATE ka_benutzergruppe_tbl SET 
	name = new.name 
	WHERE ka_benutzergruppe_tbl.id = new.id;

GRANT SELECT ON ka_benutzergruppe_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_benutzergruppe_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_benutzergruppe_tbl_view TO GROUP :admin_ka_group;

-- 
-- Sicht auf die Benutzer-Gruppen Transitionstabelle
--

CREATE OR REPLACE VIEW nm_benutzer_gruppe_tbl_view AS 
SELECT nm_benutzer_gruppe_tbl.id, nm_benutzer_gruppe_tbl.benutzer_id,
nm_benutzer_gruppe_tbl.gruppe_id 
FROM nm_benutzer_gruppe_tbl;

CREATE OR REPLACE RULE nm_benutzer_gruppe_delete_rule AS ON DELETE TO 
nm_benutzer_gruppe_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE nm_benutzer_gruppe_insert_rule AS ON INSERT TO
nm_benutzer_gruppe_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE nm_benutzer_gruppe_update_rule AS ON UPDATE TO
nm_benutzer_gruppe_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON nm_benutzer_gruppe_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON nm_benutzer_gruppe_tbl_view TO GROUP :admin_ka_group;

-- Sicht auf die KA Konfigurations Tabelle 

CREATE OR REPLACE VIEW ka_konfiguration_tbl_view AS 
SELECT ka_konfiguration_tbl.id, ka_konfiguration_tbl.bez,
ka_konfiguration_tbl.wert FROM ka_konfiguration_tbl;

CREATE OR REPLACE RULE ka_konfiguration_delete_rule AS ON DELETE TO 
ka_konfiguration_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_konfiguration_insert_rule AS ON INSERT TO 
ka_konfiguration_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_konfiguration_update_rule AS ON UPDATE TO ka_konfiguration_tbl_view 
	DO INSTEAD UPDATE ka_konfiguration_tbl SET
	wert = new.wert
	WHERE bez = new.bez;

GRANT SELECT ON ka_konfiguration_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_konfiguration_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_konfiguration_tbl_view TO GROUP :admin_ka_group;

-- 
-- Sicht auf die Dokumenten Tabelle 
--

CREATE OR REPLACE VIEW ka_global_dokumente_tbl_view AS 
SELECT ka_dokumente_tbl.id, ka_dokumente_tbl.master_id,
ka_dokumente_tbl.groesse, ka_dokumente_tbl.mime, ka_dokumente_tbl.daten, ka_dokumente_tbl.name 
FROM ka_dokumente_tbl WHERE ka_dokumente_tbl.master_id is NULL;

CREATE OR REPLACE RULE ka_global_dokumente_delete_rule AS ON DELETE TO 
ka_global_dokumente_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_global_dokumente_insert_rule AS ON INSERT TO 
ka_global_dokumente_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_global_dokumente_update_rule AS ON UPDATE TO ka_global_dokumente_tbl_view 
	DO INSTEAD  UPDATE ka_dokumente_tbl SET 
	groesse = new.groesse, 
	mime = new.mime, 
	daten = new.daten, 
	name = new.name 
	WHERE ka_dokumente_tbl.id = new.id;

GRANT SELECT ON ka_global_dokumente_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_global_dokumente_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_global_dokumente_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW ka_fall_dokumente_tbl_view AS 
SELECT ka_dokumente_tbl.id, ka_dokumente_tbl.uuid_id, ka_dokumente_tbl.master_id,
ka_dokumente_tbl.groesse, ka_dokumente_tbl.mime, ka_dokumente_tbl.daten,
ka_dokumente_tbl.name FROM ka_dokumente_tbl 
INNER JOIN master_tbl ON
ka_dokumente_tbl.master_id = master_tbl.id 
WHERE master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
	    SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
	    WHERE nm2.benutzer_id = getuserid()
	)
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE ka_fall_dokumente_delete_rule AS ON DELETE TO 
ka_fall_dokumente_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_fall_dokumente_insert_rule AS ON INSERT TO 
ka_fall_dokumente_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_fall_dokumente_update_rule AS ON UPDATE TO ka_fall_dokumente_tbl_view 
	DO INSTEAD  UPDATE ka_dokumente_tbl SET 
	groesse = new.groesse, 
	mime = new.mime, 
	daten = new.daten, 
	name = new.name 
	WHERE ka_dokumente_tbl.id = new.id;

GRANT SELECT ON ka_fall_dokumente_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON ka_fall_dokumente_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_fall_dokumente_tbl_view TO GROUP :admin_ka_group;

-- 
-- Sicht auf die Termine Tabelle 
--

CREATE OR REPLACE VIEW ka_global_termine_tbl_view AS 
SELECT ka_termine_tbl.id, ka_termine_tbl.master_id, ka_termine_tbl.sdatum, 
ka_termine_tbl.edatum, ka_termine_tbl.name, ka_termine_tbl.beschreibung, ka_termine_tbl.art
FROM ka_termine_tbl WHERE ka_termine_tbl.master_id is NULL;

CREATE OR REPLACE RULE ka_global_termine_delete_rule AS ON DELETE TO 
ka_global_termine_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_global_termine_insert_rule AS ON INSERT TO 
ka_global_termine_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_global_termine_update_rule AS ON UPDATE TO ka_global_termine_tbl_view 
	DO INSTEAD  UPDATE ka_termine_tbl SET 
	sdatum = new.sdatum, 
	edatum = new.edatum, 
	name = new.name,
	beschreibung = new.beschreibung,
	art = new.art
	WHERE ka_termine_tbl.id = new.id;

GRANT SELECT ON ka_global_termine_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_global_termine_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_global_termine_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW ka_fall_termine_tbl_view AS 
SELECT ka_termine_tbl.id, ka_termine_tbl.master_id,
ka_termine_tbl.sdatum, ka_termine_tbl.edatum,
ka_termine_tbl.name, ka_termine_tbl.beschreibung, ka_termine_tbl.art FROM ka_termine_tbl 
INNER JOIN master_tbl ON
ka_termine_tbl.master_id = master_tbl.id 
WHERE master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
	    SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
	    WHERE nm2.benutzer_id = getuserid()
	)
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE ka_fall_termine_delete_rule AS ON DELETE TO 
ka_fall_termine_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_fall_termine_insert_rule AS ON INSERT TO 
ka_fall_termine_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_fall_termine_update_rule AS ON UPDATE TO ka_fall_termine_tbl_view 
	DO INSTEAD  UPDATE ka_termine_tbl SET 
	sdatum = new.sdatum, 
	edatum = new.edatum, 
	name = new.name,
	beschreibung = new.beschreibung,
	art = new.art
	WHERE ka_termine_tbl.id = new.id;

GRANT SELECT ON ka_fall_termine_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON ka_fall_termine_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_fall_termine_tbl_view TO GROUP :admin_ka_group;

-- Sicht auf die KA Status Tabelle 

CREATE OR REPLACE VIEW ka_status_tbl_view AS 
SELECT
	ka_status_tbl.id,
	ka_status_tbl.master_id,
	ka_status_tbl.status, 
	ka_status_tbl.zugriff
FROM ka_status_tbl;

CREATE OR REPLACE RULE ka_status_delete_rule AS ON DELETE TO 
ka_status_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_status_insert_rule AS ON INSERT TO 
ka_status_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_status_update_rule AS ON UPDATE TO ka_status_tbl_view 
	DO INSTEAD UPDATE ka_status_tbl SET
	status  = new.status,
	zugriff = new.zugriff
WHERE id = new.id;

GRANT SELECT ON ka_status_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON ka_status_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_status_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_status_tbl_view TO GROUP :admin_ka_group;

--
-- Views auf dem Logbuch
--

CREATE OR REPLACE VIEW ka_logbuch_tbl_view 
    AS SELECT
        ka_logbuch_tbl.id         AS id,
        ka_logbuch_tbl.uuid_id    AS uuid_id,
        ka_logbuch_tbl.master_id  AS master_id,
        ka_logbuch_tbl.datum      AS datum,
        ka_logbuch_tbl.bearbeiter AS bearbeiter,
        ka_logbuch_tbl.art        AS art,
        ka_logbuch_tbl.kurz_notiz AS kurz_notiz,
        ka_logbuch_tbl.notiz      AS notiz,
        ka_logbuch_tbl.dauer      AS dauer
    FROM ka_logbuch_tbl
    INNER JOIN master_tbl ON
        ka_logbuch_tbl.master_id = master_tbl.id
    WHERE
        master_tbl.bearbeiter_id = getuserid()
        OR master_tbl.id IN (
            SELECT nm.master_id FROM nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id IN (
                SELECT nm2.gruppe_id FROM nm_benutzer_gruppe_tbl nm2 
                WHERE nm2.benutzer_id = getuserid())
        )
        OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE ka_logbuch_update_rule
    AS ON UPDATE TO ka_logbuch_tbl_view 
	DO INSTEAD
        UPDATE ka_logbuch_tbl SET
            datum      = new.datum,
            bearbeiter = new.bearbeiter,
            art        = new.art,
            kurz_notiz = new.kurz_notiz,
            notiz      = new.notiz,
            dauer      = new.dauer
	    WHERE ka_logbuch_tbl.id = new.id;

CREATE OR REPLACE RULE ka_logbuch_delete_rule 
    AS ON DELETE TO  ka_logbuch_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_logbuch_insert_rule
    AS ON INSERT TO ka_logbuch_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON ka_logbuch_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :admin_ka_group;

--
-- Views auf die Nachrichten
--

CREATE OR REPLACE VIEW ka_nachrichten_art_tbl_view 
    AS SELECT
        a.id           AS id,
        a.beschreibung AS beschreibung
    FROM ka_nachrichten_art_tbl a;

CREATE OR REPLACE RULE ka_nachrichten_art_update_rule 
    AS ON UPDATE TO ka_nachrichten_art_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_nachrichten_art_delete_rule 
    AS ON DELETE TO ka_nachrichten_art_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_nachrichten_art_insert_rule
    AS ON INSERT TO ka_nachrichten_art_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON ka_nachrichten_art_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_nachrichten_art_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW ka_nachrichten_tbl_view 
    AS SELECT
        n.id           AS id,
        n.datum        AS datum,
        n.art          AS art,
        a.beschreibung AS art_beschreibung,
        n.titel        AS titel,
        n.nachricht    AS nachricht
    FROM ka_nachrichten_tbl n JOIN ka_nachrichten_art_tbl a
    ON n.art = a.id;

CREATE OR REPLACE RULE ka_nachrichten_update_rule
    AS ON UPDATE TO ka_nachrichten_tbl_view 
	DO INSTEAD
        UPDATE ka_nachrichten_tbl SET
            art        = new.art,
            titel      = new.titel,
            nachricht  = new.nachricht
	    WHERE ka_nachrichten_tbl.id = new.id;

CREATE OR REPLACE RULE ka_nachrichten_delete_rule 
    AS ON DELETE TO  ka_nachrichten_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_nachrichten_insert_rule
    AS ON INSERT TO ka_nachrichten_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON ka_nachrichten_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_nachrichten_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_nachrichten_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE  VIEW nm_benutzer_nachrichten_tbl_view AS
    SELECT id, benutzer_id, nachrichten_id, gelesen FROM nm_benutzer_nachrichten_tbl; 

GRANT SELECT ON nm_benutzer_nachrichten_tbl_view TO :cm_ka_group;
GRANT SELECT ON nm_benutzer_nachrichten_tbl_view TO :admin_ka_group;

COMMIT;
