BEGIN;
--
-- Einfuehrung von expliziten Phasen des Case-Managements
--
SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
        VALUES ('update_1_0_4/02_phasen.sql', current_timestamp);

-- Tabelle mit den Beschreibungen der Phasen
CREATE TABLE ka_phasen_tbl (
    id          SERIAL PRIMARY KEY NOT NULL,
    description VARCHAR(128)       NOT NULL
);

INSERT INTO ka_phasen_tbl (id, description) VALUES (-1, 'Unbekannt');
INSERT INTO ka_phasen_tbl (id, description) VALUES ( 0, 'Beratungskunde/-kundin');
INSERT INTO ka_phasen_tbl (id, description) VALUES ( 1, 'Im Case-Management');
INSERT INTO ka_phasen_tbl (id, description) VALUES ( 2, 'Kompetenzfeststellung');
INSERT INTO ka_phasen_tbl (id, description) VALUES ( 3, 'Förderphase');
INSERT INTO ka_phasen_tbl (id, description) VALUES ( 4, 'Nachbetreuung');
INSERT INTO ka_phasen_tbl (id, description) VALUES ( 5, 'CM erfolgreich');
INSERT INTO ka_phasen_tbl (id, description) VALUES ( 6, 'CM abgebrochen');
INSERT INTO ka_phasen_tbl (id, description) VALUES ( 7, 'Beratung abgebrochen');
INSERT INTO ka_phasen_tbl (id, description) VALUES ( 8, 'Beratung erfolgreich');

-- weitere Spalte in der ka_status_tbl

ALTER TABLE ka_status_tbl ADD COLUMN phase INTEGER DEFAULT -1;

ALTER TABLE ka_status_tbl ADD CONSTRAINT ka_status_tbl_phase_fkey 
    FOREIGN KEY (phase) REFERENCES ka_phasen_tbl (id) ON DELETE SET DEFAULT;

-- passende View/Rules neu aufbauen

DROP VIEW ka_status_tbl_view;

CREATE OR REPLACE VIEW ka_status_tbl_view AS 
SELECT 
    ka_status_tbl.id, 
    ka_status_tbl.master_id,
    ka_status_tbl.status, 
    ka_status_tbl.zugriff,
    ka_status_tbl.phase
FROM ka_status_tbl;

CREATE OR REPLACE RULE ka_status_delete_rule AS ON DELETE TO 
ka_status_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_status_insert_rule AS ON INSERT TO 
ka_status_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_status_update_rule AS 
ON UPDATE TO ka_status_tbl_view 
DO INSTEAD UPDATE ka_status_tbl SET
    status  = new.status,
    zugriff = new.zugriff,
    phase   = new.phase
WHERE id = new.id;

GRANT SELECT ON ka_status_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON ka_status_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_status_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_status_tbl_view TO GROUP :admin_ka_group;

COMMIT;
