-- Author: Torsten Irlaender
-- Funktionen, die für die Anonymisierung der Fallakten genutzt werden.
BEGIN;

SET ROLE :adm_ka_owner;

CREATE OR REPLACE FUNCTION shorten_value(myvalue varchar, len integer) 
RETURNS varchar 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    return substring(myvalue, 1, len);
END;
$$;

REVOKE ALL ON FUNCTION shorten_value(varchar, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION shorten_value(varchar, integer) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION shorten_value(varchar, integer) TO GROUP :admin_ka_group;

CREATE OR REPLACE FUNCTION hash_value(myvalue varchar, myvalue2 varchar) 
RETURNS varchar 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    return md5(coalesce(myvalue, '') || coalesce(myvalue2, ''));
END;
$$;

REVOKE ALL ON FUNCTION hash_value(varchar, varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION hash_value(varchar, varchar) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION hash_value(varchar, varchar) TO GROUP :admin_ka_group;

CREATE OR REPLACE FUNCTION get_age(myvalue date) 
RETURNS date 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    return date_trunc('year', myvalue::timestamp);
END;
$$;

REVOKE ALL ON FUNCTION get_age(date) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION get_age(date) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION get_age(date) TO GROUP :admin_ka_group;

CREATE OR REPLACE FUNCTION check_bool(myvalue integer, mybool integer) 
RETURNS integer 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    IF mybool = 1 THEN
        return myvalue;
    ELSE
        return NULL::integer;
    END IF;
END;
$$;

REVOKE ALL ON FUNCTION check_bool(integer, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION check_bool(integer, integer) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION check_bool(integer, integer) TO GROUP :admin_ka_group;

CREATE OR REPLACE FUNCTION check_bool(myvalue varchar, mybool integer) 
RETURNS varchar 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    IF mybool = 1 THEN
        return myvalue;
    ELSE
        return NULL::varchar;
    END IF;
END;
$$;

REVOKE ALL ON FUNCTION check_bool(varchar, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION check_bool(varchar, integer) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION check_bool(varchar, integer) TO GROUP :admin_ka_group;

-- Nun folgen die Views die die Daten anonymisiert herausgeben

CREATE OR REPLACE VIEW master_tbl_anonym_view
AS SELECT
    id,
    uuid_id,
    bearbeiter_id,
    einverstaendniserklaerung,
    NULL::VARCHAR(60) AS fkz,
    NULL::VARCHAR(60) AS name_kompetenzagentur,
    NULL::VARCHAR(60) AS vorname,
    hash_value(name, vorname) AS name,
    erstgespraech,
    zugang_kompetenzagentur,
    NULL::VARCHAR(100) AS kundennummer,
    schulpflichtig,
    noch_schueler,
    bei_eintritt_ist_der_jugendliche,
    rechtlicher_kontext,
    schnittstelle_zu,
    NULL::VARCHAR(100) AS kontaktdaten_arge,
    NULL::VARCHAR(100) AS kundennummer_arge,
    NULL::VARCHAR(100) AS kontaktdaten_agentur_fuer_arbeit,
    NULL::VARCHAR(100) AS kundennummer_agentur_fuer_arbeit,
    NULL::VARCHAR(100) AS kundennummer_jugendamt,
    NULL::VARCHAR(100) AS kontaktdaten_jugendamt,
    NULL::VARCHAR(100) AS kontaktdaten_schule,
    NULL::VARCHAR(100) AS schulname,
    NULL::VARCHAR(100) AS kontaktdaten_gerichtshilfe,
    NULL::VARCHAR(30) AS kundennummer_gerichtshilfe,
    NULL::TEXT AS kontaktdaten_weitere_institutionen,
    NULL::VARCHAR(60) AS vorname_1,
    NULL::VARCHAR(60) AS name_1,
    geschlecht,
    get_age(geburtsdatum) AS geburtsdatum,
    familienstand,
    NULL::VARCHAR(200) AS strasse,
    shorten_value(plz, 3) AS plz,
    NULL::VARCHAR(60) AS wohnort,
    groesse_wohnort,
    NULL::VARCHAR(60) AS vorwahl,
    NULL::VARCHAR(60) AS telefon_1,
    NULL::VARCHAR(120) AS mobiltelefon,
    NULL::VARCHAR(120) AS email,
    NULL::TEXT AS telefon_sonstige,
    NULL::VARCHAR(120) AS zweiter_wohnort,
    geburtsland,
    NULL::VARCHAR(60) AS geburtsland_sonstige_nennung,
    muttersprache,
    NULL::VARCHAR(60) AS muttersprache_sonstige_nennung,
    staatsangehoerigkeit,
    NULL::VARCHAR(60) AS staatsangehoerigkeit_sonstige_nennung,
    aufenthaltsstatus,
    aufenthaltsdauer,
    vater,
    mutter,
    familiaere_situation_1,
    NULL::VARCHAR(60) AS vorname_2,
    NULL::VARCHAR(60) AS name_2,
    NULL::VARCHAR(60) AS vorwahl_1,
    NULL::VARCHAR(60) AS telefon_1_1,
    NULL::VARCHAR(60) AS mobiltelefon_1,
    NULL::VARCHAR(120) AS email_1,
    geschwister,
    anzahl_kinder,
    haushalt_jugendlicher,
    alleinerziehend,
    familie_selbststaendig,
    familie_staatliche_unterstuetzung,
    jugendlicher_selbststaendig,
    jugendlicher_staatliche_unterstuetzung,
    jugendlicher_unterstuetzung_soziales_umfeld,
    wohnaufenthalt,
    arzt_gutachten_physisch,
    check_bool(physische_beeintraechtigungen, arzt_gutachten_physisch) AS physische_beeintraechtigungen,
    check_bool(physische_beeintraechtigungen_text, arzt_gutachten_physisch) AS physische_beeintraechtigungen_text,
    arzt_gutachten_psychisch,
    check_bool(psychische_beeintraechtigungen, arzt_gutachten_psychisch) AS psychische_beeintraechtigungen,
    check_bool(psychische_beeintraechtigungen_text, arzt_gutachten_psychisch) AS psychische_beeintraechtigungen_text,
    mutter_2,
    vater_2,
    grosseltern,
    geschwister_2,
    pflegefamilie_1,
    stiefmutter,
    stiefvater,
    verwandte_1,
    partner,
    heimbetreuerin,
    betreuerin,
    lehrerin,
    mitbewohner,
    partner_mutter,
    partner_vater,
    freund,
    keine_bezugsperson,
    hobbies_1,
    hobbies_text,
    mitarbeit_organisationen,
    organisationen_vereine_text,
    sonstiges_1,
    sonstiges_text,
    clique,
    freundin,
    parterin,
    schule_2,
    klassenstufe,
    klassenwiederholung,
    abgang,
    nicht_anwesende_tage,
    unentschuldigte_tage,
    schulabschluss_1,
    schulabschlusserwerbsland,
    NULL::VARCHAR(60) AS schulabschlusserwerbsland_sonstige_nennung,
    abschluss_betriebliche_ausbildung,
    abschluss_im_ausbildungsberuf,
    ausbildungsdauer,
    wann_abgeschlossen,
    weitere_ausbildungen,
    weitere_ausbildungen_text,
    dauer_weitere_ausbildungen,
    abgebrochene_ausbildungen,
    NULL::TEXT AS abgebrochene_ausbildungen_text,
    abschluss_ausserbetriebliche_ausbildung,
    abschluss_im_ausbildungsberuf_1,
    ausbildungsdauer_1,
    wann_abgeschlossen_1,
    weitere_ausbildungen_1,
    NULL::TEXT AS weitere_ausbildungen_text_1,
    dauer_weitere_ausbildungen_1,
    abgebrochene_ausbildungen_1,
    NULL::TEXT AS abgebrochene_ausbildungen_text_1,
    massnahmeteilnahme,
    NULL::VARCHAR(60) AS letzte_massnahme,
    finanzierung,
    austrittsdatum,
    dauer,
    weitere_massnahmen,
    massnahme_1,
    massnahme1_finanzierung,
    massnahme_2,
    massnahme2_finanzierung,
    massnahme_3,
    massnahme3_finanzierung,
    abgebrochene_massnahmen,
    erwerbstaetigkeit_job,
    NULL::VARCHAR(60) AS letzte_taetigkeit,
    dauer_1,
    austrittsdatum_1,
    weitere_taetigkeiten,
    weitere_jobs_text,
    dauer_weitere_taetigkeiten,
    arbeitslosigkeit_1,
    anzahl_arbeitslosigkeit,
    dauer_nach_letzter_beschaeftigung,
    dauer_insgesamt,
    wehrpflicht_zivildienst_freiwilligenjahr_1,
    art_des_dienstes,
    dauer_2,
    mutterschutz_erziehungsurlaub,
    anzahl,
    dauer_letzter_erziehungsurlaub,
    dauer_insgesamt_1,
    kompetenzfeststellung,
    zielsetzung,
    begleitung,
    welche_vermittlung_nennung,
    NULL::TEXT AS freie_dokumentation,
    bildender_bereich,
    bereich_berufliche_qualifizierung,
    bereich_berufsvorbereitung,
    bereich_lebensbewaeltigung,
    sonstiger_foerderbedarf,
    NULL::VARCHAR(120) AS sonstiger_foerderbedarf_nennung,
    foerderzeitraum_start,
    foerderzeitraum_ende,
    NULL::TEXT AS dokumentation_zum_foerderplan,
    anzahl_kontakte_termine,
    anzahl_der_stunden,
    zeitraum_fallmanagement,
    beendigung_5,
    art_beendigung,
    abbruch_durch,
    gruende_abbruch,
    NULL::VARCHAR(60) AS gruende_sonstiges_nennung_4,
    vermittlung,
    vermittlung_nennung,
    veraenderungen_rechtskreis,
    allgemein_bildender_schulabschluss,
    allgemein_bildender_schulabschluss_sonstiges_nennung,
    massnahmeabschluss,
    schulart,
    schule_sonstiges_nennung,
    NULL::INTEGER AS klassenstufe_1,
    ausbildung,
    ausbildung_sonstiges_nennung,
    umschulung,
    umschulung_nennung,
    arbeitstaetigkeit,
    erwerbstatus,
    erwerbsstatus_sonstiges_nennung,
    beschaeftigungsumfang,
    kategorien_nichterwerbsarbeit_nichtausbildung,
    kategorien_sonstiges_nennung,
    zeitraum_nachbetreuung,
    verbleib_nachbetreuung,
    mitgliedschaft_organisationen_vereinen,
    intensitaet_mitgliedschaft,
    nutzung_kulturangeboten,
    haeufigkeit,
    soziales_engagement,
    unternehmungen_freunden,
    unternehmungen_familie,
    peergroup,
    familie_1,
    beziehung,
    arbeit_schule,
    bester_freund_freundin,
    einschaetzung_zielerreichung,
    erreichte_qualifikation,
    integration_ist_nachhaltig,
    selbststaendige_berufswegplanung,
    eigenstaendige_lebensgestaltung,
    soziale_bezugssysteme,
    teilnahme_gesellschaftliches_leben,
    NULL::TEXT AS freie_dokumentation_abschlussbewertung FROM master_tbl_view;

CREATE OR REPLACE RULE master_tbl_anonym_delete_rule AS ON DELETE TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE master_tbl_anonym_insert_rule AS ON INSERT TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON master_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON master_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_kompetenzfestellung_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    verfahren_kompetenzfeststellung,
    externeranbieter,
    assessment_center,
    interview,
    biografieorientierte_methoden,
    selbsttest_selbstbeurteilung,
    sonstiges_2,
    artdesverfahrens_nennung_sonstiges,
    testverfahren,
    intelligenztest,
    intelligenztest_nennung,
    schultest,
    schultest_nennung,
    leistungstest,
    leistungstest_nennung,
    persoenlichkeitstest,
    persoenlichkeitstest_nennung,
    einstellungs_interessentest,
    einstellungs_interessentest_nennung,
    dauer_des_verfahrens,
    rechenfaehigkeit,
    raeumliches_vorstellungsvermoegen,
    grundverstaendnis_naturwissenschaftlicher_zusammenhaenge,
    logisches_denken,
    wirtschaftliche_grundkenntnisse,
    allgemeine_pc_kenntnisse,
    spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung,
    internet_anwendungen_und_e_mail_kenntnisse,
    andere_medien,
    hand_fingergeschick,
    formauffassung,
    umgang_maschinen_werkzeugen,
    gestalterische_fantasie,
    methoden_der_informationsgewinnung,
    methoden_der_informationsverarbeitung,
    einsatz_lern_und_arbeitstechniken,
    problemloesungsorientierung,
    texte_lesen_und_verstehen,
    texte_schreiben_und_formulieren,
    deutsche_sprache,
    sprache1,
    sprache1_sonstige_nennung,
    beherrschung_sprache1,
    sprache2,
    sprache2_sonstige_nennung,
    beherrschung_sprache2,
    sprache3,
    sprache3_sonstige_nennung,
    beherrschung_sprache3,
    kommunikationsfaehigkeit,
    kritikfaehigkeit,
    konfliktfaehigkeit,
    kreativitaet,
    team_und_kooperationsfaehigkeit,
    solidarisches_verhalten,
    selbstaendiges_lernen_arbeiten,
    selbstreflexion_selbsteinschaetzung,
    verantwortungsuebernahme,
    zielorientierung,
    zuverlaessigkeit,
    altersgerechter_entwicklungsstand,
    gesundheitliche_voraussetzungen,
    konzentration,
    ausdauer,
    durchhaltevermoegen,
    merkfaehigkeit,
    bearbeitungsgeschwindigkeit,
    befaehigung_zu_daueraufmerksamkeit,
    leistungsbereitschaft,
    sorgfalt,
    umgangsformen,
    NULL::TEXT AS dokumentation_kompetenzfeststellung FROM rg_kompetenzfestellung_tbl_view;

CREATE OR REPLACE RULE rg_kompetenzfestellung_tbl_anonym_delete_rule AS ON DELETE TO rg_kompetenzfestellung_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_kompetenzfestellung_tbl_anonym_insert_rule AS ON INSERT TO rg_kompetenzfestellung_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_kompetenzfestellung_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_kompetenzfestellung_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_bildenden_bereich_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    angebotsform,
    art_der_unterstuetzung,
    durchfuehrende_institution,
    unterstuetzung_start,
    unterstuetzung_ende,
    zeitlicher_umfang_unterstuetzung,
    beendigung_1,
    beendigung_durch,
    gruende_vorzeitige_beendigung,
    NULL::VARCHAR(120) AS gruende_sonstiges_nennung,
    nachweis_bescheinigung,
    NULL::VARCHAR(120) AS sonstiges_nachweis_bescheinigung FROM rg_angebote_bildenden_bereich_tbl_view;

CREATE OR REPLACE RULE rg_angebote_bildenden_bereich_tbl_anonym_delete_rule AS ON DELETE TO rg_angebote_bildenden_bereich_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_bildenden_bereich_tbl_anonym_insert_rule AS ON INSERT TO rg_angebote_bildenden_bereich_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_bildenden_bereich_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_bildenden_bereich_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_berufsvorbereitung_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    angebotsform_1,
    art_der_unterstuetzung_1,
    unterstuetzung_sonstiges_nennung,
    durchfuehrende_institution_1,
    unterstuetzung_start_1,
    unterstuetzung_ende_1,
    zeitlicher_umfang_unterstuetzung_1,
    beendigung_2,
    beendigung_durch_1,
    gruende_vorzeitige_beendigung_1,
    NULL::VARCHAR(120) AS gruende_sonstiges_nennung_1,
    nachweis_bescheinigung_1,
    NULL::VARCHAR(120) AS sonstiges_nachweis_bescheinigung_1 FROM rg_angebote_berufsvorbereitung_tbl_view;

CREATE OR REPLACE RULE rg_angebote_berufsvorbereitung_tbl_anonym_delete_rule AS ON DELETE TO rg_angebote_berufsvorbereitung_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_berufsvorbereitung_tbl_anonym_insert_rule AS ON INSERT TO rg_angebote_berufsvorbereitung_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_berufsvorbereitung_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_berufsvorbereitung_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_berufliche_qualifizierung_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    angebotsform_2,
    angebote_im_bereich,
    unterstuetzung_sonstiges_nennung_1,
    durchfuehrende_institution_2,
    unterstuetzung_start_2,
    unterstuetzung_ende_2,
    zeitlicher_umfang_unterstuetzung_2,
    beendigung_3,
    beendigung_durch_2,
    gruende_vorzeitige_beendigung_2,
    NULL::VARCHAR(120) AS gruende_sonstiges_nennung_2,
    nachweis_bescheinigung_2,
    NULL::VARCHAR(120) AS sonstiges_nachweis_bescheinigung_2 FROM rg_angebote_berufliche_qualifizierung_tbl_view;

CREATE OR REPLACE RULE rg_angebote_berufliche_qualifizierung_tbl_anonym_delete_rule AS ON DELETE TO rg_angebote_berufliche_qualifizierung_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_berufliche_qualifizierung_tbl_anonym_insert_rule AS ON INSERT TO rg_angebote_berufliche_qualifizierung_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_berufliche_qualifizierung_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_berufliche_qualifizierung_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_lebensbewaeltigung_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    angebotsform_3,
    unterstuetzung_angebotsform_sonstiges_nennung,
    art_der_unterstuetzung_2,
    unterstuetzung_sonstiges_nennung_2,
    durchfuehrende_institution_3,
    unterstuetzung_start_3,
    unterstuetzung_ende_3,
    zeitlicher_umfang_unterstuetzung_3,
    beendigung_4,
    beendigung_durch_3,
    gruende_vorzeitige_beendigung_3,
    NULL::VARCHAR(120) AS gruende_sonstiges_nennung_3,
    nachweis_bescheinigung_3,
    NULL::VARCHAR(120) AS sonstiges_nachweis_bescheinigung_3 FROM rg_angebote_lebensbewaeltigung_tbl_view;

CREATE OR REPLACE RULE rg_angebote_lebensbewaeltigung_tbl_anonym_delete_rule AS ON DELETE TO rg_angebote_lebensbewaeltigung_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_lebensbewaeltigung_tbl_anonym_insert_rule AS ON INSERT TO rg_angebote_lebensbewaeltigung_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_lebensbewaeltigung_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_lebensbewaeltigung_tbl_anonym_view TO GROUP :admin_ka_group;

COMMIT;
