# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging
from paste.httpexceptions           import HTTPNotFound

from waskaweb.controllers.CaseBase  import CasebaseController 
from waskaweb.controllers.case      import CaseController 
from waskaweb.model.repeatgroup     import AidList 
from waskaweb.lib.security          import checkRole
from waskaweb.lib.renderer          import ViewRenderer
from waskaweb.lib.base              import *

log = logging.getLogger(__name__)

class NavigationController(CasebaseController):

    def __before__(self):
        CasebaseController.__before__(self)

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def index(self):
        # Return a rendered template
        #   return render('/some/template.mako')
        # or, Return a response
        return 'Hello World'

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def toggle(self, ds_id, page_id):
        '''This function toggles the selected item in thenavigation tree and
        than returns the last selected page. If no page was selected (e.g fresh
        opened case) it will return the rendered page of the item which will be
        toggeled'''

        # Check values and toggle navigation tree
        ds_id        = self._checkInt(ds_id)
        page_id      = self._checkInt(page_id)
        try:
            ti = self.navigation.getTreeItem(page_id)
        except:
            return render('/casemanagement/formular.mako')
        ti.toggle()

        # Load last selected page
        sti = ti 
        session_case = session.get('case')
        if session_case.id != ds_id: #Case seems to have changed. 
            redirect_to(controller="/case", action="select", id=ds_id, confirmed=0)
            sti = self.navigation.getTreeItem(session_case.getPageId())
        c.ds_id      = ds_id
        real_id      = sti.realId(ds_id)
        c.case       = self._loadCase(ds_id)
        c.page_id    = session_case.getPageId() or page_id
        session.save()

        c.print_form = "show" # Which page to open in popup on printing?
        read_only = (session_case.getMode() == "show")
        content, navigation = self._getRenderdFormElements(sti, ds_id, sti.key, ro_mode=read_only)
        c.formular = content
        c.form_navigation = navigation 
        return render('/casemanagement/formular.mako')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
