# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

from types import StringTypes, IntType, LongType

from pyExcelerator import Workbook

from exportselection import CasePartsSelection, SelectorFactory

FETCH_ALL_IDS = \
"""SELECT id FROM master_tbl_view"""

RG_VIEWS = (
    ("Kompetenzfeststellung", "kompetenzfestellung"),
    ("Angebote BQ",           "angebote_berufliche_qualifizierung"),
    ("Angebote BV",           "angebote_berufsvorbereitung"),
    ("Angebote BB",           "angebote_bildenden_bereich"),
    ("Angebote LB",           "angebote_lebensbewaeltigung"))

BLACK_LIST = set(['id', 'master_id', 'uuid_id', 'bearbeiter_id'])


def to_str(x):
    t = type(x)
    if t in (IntType, LongType):
        return x
    if t in StringTypes:
        x = unicode(str(x), "utf-8")
        if len(x) > 32765: x = x[:32765]
        return x
    return str(x)

class Sheet:

    def __init__(self, worksheet, relation):
        self.worksheet    = worksheet
        self.relation     = relation
        self.row          = 1
        self.column_names = None
        self.column       = 0
        self.max_column   = 0

    def next_row(self):
        if self.column > self.max_column:
            self.max_column = self.column
        self.column = 0
        self.row += 1

    def add_dataset(self, dataset):
        if self.column_names is None:
            self.column_names = [x[0] for x in dataset]

        column    = self.column
        row       = self.row
        worksheet = self.worksheet
        for x in dataset:
            x = x[1]
            if not x is None:
                worksheet.write(row, column, to_str(x))
            column += 1
        self.column = column

    def generate_header(self):
        if self.column_names is None:
            return

        num_names = max(1, len(self.column_names))
        repeats = self.max_column / num_names

        worksheet = self.worksheet

        j = 0
        for i in xrange(repeats):
            suffix = i != 0 and ("$%d" % i) or ""
            for name in self.column_names:
                worksheet.write(0, j, "%s%s" % (name, suffix))
                j += 1

    def add_row(self, row):
        columns = row._index.items() # XXX: hackish!
        columns.sort(lambda a, b: cmp(a[1], b[1]))
        self.add_dataset([(a[0], row[a[1]]) for a in columns if a[0] not in BLACK_LIST])

    def fetch_data(self, mid, cur, selector):
        stmnt = selector.getSelectStatementByView(self.relation)
        if not stmnt:
            return
        cur.execute(stmnt, { 'id': mid })
        while True:
            row = cur.fetchone()
            if not row: break
            self.add_row(row)

class Master(Sheet):
    def __init__(self, worksheet, relation):
        Sheet.__init__(self, worksheet, relation)

    def fetch_data(self, mid, cur, selector):
        stmnt = selector.getSelectStatementByView(self.relation)
        if not stmnt: return
        cur.execute(stmnt, { 'id': mid })
        row = cur.fetchone()
        if not row: return
        self.add_row(row)

class Book:

    def __init__(self, selector):
        self.selector = selector

        self.workbook  = Workbook()

        sheets = [Master(self.workbook.add_sheet("Hauptdatensatz"), 'master_tbl')]

        for rg in RG_VIEWS:
            sheets.append(Sheet(self.workbook.add_sheet(rg[0]), rg[1]))

        self.sheets = sheets

    def add_cases(self, case_ids, cur):
        for case_id in case_ids:
            for sheet in self.sheets:
                sheet.fetch_data(case_id, cur, self.selector)
                sheet.next_row()

    def generate_headers(self):
        for sheet in self.sheets:
            sheet.generate_header()

    def write_xls(self, fname):
        self.workbook.save(fname)

def exportAsXLS(cur, fname, ids=None, case_parts=None, anonymize=False):
    if ids is None:
        ids = []
        cur.execute(FETCH_ALL_IDS)
        while True:
            row = cur.fetchone()
            if not row: break
            ids.append(int(row[0]))

    if case_parts is None:
        case_parts = CasePartsSelection()

    selectFactory = SelectorFactory(case_parts, anonymize)

    selector = selectFactory.createSelector()

    book = Book(selector)
    book.add_cases(ids, cur)
    book.generate_headers()
    book.write_xls(fname)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
