# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Torsten <irlaender@intevation.de>
#
import psycopg2.extras
import os
from waskaweb.model.statement import * 

import traceback
import sys

from waskaweb.lib.db import db

FETCH_SETTINGS_SQL = """SELECT bez, wert from ka_konfiguration_tbl_view"""
STORE_SETTINGS_SQL = """UPDATE ka_konfiguration_tbl_view SET wert = %(wert)s WHERE bez = %(bez)s"""

class Agency:
    def __init__(self):
        self.conf = {}
        self._fetchData()

    def _fetchData(self):
        conn, cur = None, None
        try:
            conn = db.getConnection()
            cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cur.execute(FETCH_SETTINGS_SQL)
            rows = cur.fetchall()
            for row in rows:
                self.conf[row['bez']] = row['wert']
        finally:
            db.recycleConnection(conn, cur)

    def store(self, formdata):
        conn = None
        try:
            try:
                conn = db.getConnection()
                for key, value in formdata.iteritems():
                    # TODO: Its not necessary to fetch a new cursor each time.
                    cur = None
                    try:
                        cur = conn.cursor()
                        cur.execute(STORE_SETTINGS_SQL, {'bez': key, 'wert': value})
                        conn.commit()
                    finally:
                        if cur:
                            try: cur.close()
                            except: pass
            finally:
                db.recycleConnection(conn)
        except:
            traceback.print_exc(file=sys.stderr)

    def isAnonTransfer(self):
        anon = self.conf.get('anon_transfer', '').strip().lower()
        return anon == 'on' or anon == '1'

    def getFKZ(self):
        return unicode(str(self.conf.get('fkz', '')), 'utf-8')

    def getName(self):
       return unicode(str(self.conf.get('ka_name', '')), 'utf-8')

    def getMaxSavetime(self):
       return unicode(str(self.conf.get('max_speicherdauer', '')), 'utf-8')

    def _loadDefaultStatement(self, filename):
        # XXX: Dead ugly!
        path = os.path.join(os.path.dirname(__file__), '..', '..', 'formed')
        path_statement = os.path.join(path, filename)
        f = None
        try:
            f = open(path_statement, "r")
            statement = f.read()
        finally:
            if f:
                try: f.close()
                except: pass
        return unicode(statement, 'utf-8') 
        
    def getAidPlanStatement(self):
        form = self.conf.get('fp_formular', '')
        if len(form) > 0:
            return unicode(form, 'utf-8')
        return self._loadDefaultStatement('aid_plan_statement.html')

    def getPrivacyStatement(self):
        form = self.conf.get('ee_formular', '')
        if len(form) > 0:
            return unicode(form, 'utf-8')
        return self._loadDefaultStatement('privacy_statement.html')

    def getDiscretionStatement(self):
        form = self.conf.get('ud_formular', '')
        if len(form) > 0:
            return unicode(form, 'utf-8')
        return self._loadDefaultStatement('discretion_statement.html')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
